/*
 * tickernoti
 * Copyright (c) 2012-2013 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */



#include <stdio.h>
#include <appcore-efl.h>
#include <Ecore_X.h>
#include <syspopup.h>

#include <time.h>
#include "tickernoti-syspopup.h"

#define HAVE_ELEMENTARY_X 1
#define SYSPOPUP_PARAM_LEN 3
syspopup_handler handler = {
	.def_term_fn = NULL,
	.def_timeout_fn = NULL
};

static void __win_del(void *data, Evas_Object * obj, void *event)
{
	elm_exit();
}

static Evas_Object *__create_win(const char *name)
{
	if(!name) return NULL;
	Evas_Object *eo;
	int w;
	int h;

	eo = elm_win_add(NULL, name, ELM_WIN_DIALOG_BASIC);
	if (eo) {
		elm_win_title_set(eo, name);
		elm_win_borderless_set(eo, EINA_TRUE);
		elm_win_alpha_set(eo, EINA_TRUE);
		elm_win_raise(eo);
		evas_object_smart_callback_add(eo, "delete,request",
						   __win_del, NULL);
		ecore_x_window_size_get(ecore_x_window_root_first_get(),
					&w, &h);
		evas_object_resize(eo, w, h);
	}


	return eo;
}

static int __app_create(void *data)
{
	if(!data) return -1;
	struct appdata *ad = data;

	int r;
	int ret = -1;

	/* init internationalization */
	r = appcore_set_i18n(PACKAGE, LOCALEDIR);
	if (r != 0)
	{
		return -1;
	}

	return 0;
}

static void unload_popup(void *data)
{
	if(!data) return ;
	struct appdata *ad = data;

	if (ad->rotation_event_handler)
	{
		ecore_event_handler_del(ad->rotation_event_handler);
	}

	if (ad->layout)
	{
		evas_object_del(ad->layout);
		ad->layout = NULL;
	}
}

static int __app_terminate(void *data)
{

	if(!data) return -1;
	struct appdata *ad = data;
	int ret = -1;

	unload_popup(ad);

	if (ad->win)
	{
		evas_object_del(ad->win);
		ad->win = NULL;
	}
	if (ad->b)
	{
		ret = bundle_free(ad->b);
		if (ret != 0 )
		{
		}
		ad->b = NULL;
	}

	return 0;
}

static int __app_pause(void *data)
{
	return 0;
}

static int __app_resume(void *data)
{
	return 0;
}

static void __prt_recvd_bundle(const char *key, const char *value, void *d)
{
}

static Eina_Bool timeout_cb(void *data)
{

	elm_exit();

	return ECORE_CALLBACK_CANCEL;
}

#ifdef HAVE_ELEMENTARY_X
static void
_update_window_hints(Evas_Object *obj)
{
   Ecore_X_Window xwin;
   Ecore_X_Atom _notification_level_atom;
   int level;
   // elm_win_xwindow_get() must call after elm_win_alpha_set()
   xwin = elm_win_xwindow_get(obj);

   ecore_x_icccm_hints_set(xwin, 0, ECORE_X_WINDOW_STATE_HINT_NONE, 0, 0, 0, 0, 0);
   ecore_x_netwm_window_type_set(xwin, ECORE_X_WINDOW_TYPE_NOTIFICATION);
   ecore_x_netwm_opacity_set(xwin, 0);
   // Create atom for notification level
   _notification_level_atom = ecore_x_atom_get("_E_ILLUME_NOTIFICATION_LEVEL");

   // HIGH:150, NORMAL:100, LOW:50
   level = 100;

   // Set notification level of the window
   ecore_x_window_prop_property_set(xwin, _notification_level_atom, ECORE_X_ATOM_CARDINAL, 32, &level, 1);
}

#endif
static Evas_Object
*_create_window(Evas_Object *parent, const char *name)
{
   Evas_Object *win;

   win = elm_win_add(parent, name, ELM_WIN_BASIC);
   elm_win_title_set(win, name);
   elm_win_borderless_set(win, EINA_TRUE);
   elm_win_autodel_set(win, EINA_TRUE);
   elm_win_alpha_set(win, EINA_TRUE);
   evas_object_size_hint_weight_set(win, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
   evas_object_size_hint_align_set(win, EVAS_HINT_FILL, EVAS_HINT_FILL);

#ifdef HAVE_ELEMENTARY_X
   // set top window
   _update_window_hints(win);
#endif
   return win;
}

static void
_update_geometry_on_rotation(void *data, int *x, int *y, int *w)
{
   int angle;

   if(!data) return ;
   struct appdata *ad = data;

   angle = ad->angle;
#ifdef HAVE_ELEMENTARY_X
   Evas_Coord root_w, root_h;

   /*
   * manually calculate win_tickernoti_indi window position & size
   *  - win_indi is not full size window
   */
   ecore_x_window_size_get(ecore_x_window_root_first_get(), &root_w, &root_h);
   // rotate win
   switch(angle)
     {
      case 90:
         *w = root_h;
         if (ad->orient == _ELM_TICKERNOTI_ORIENT_BOTTOM)
           *x = root_w - ad->noti_height;
         break;
      case 270:
         *w = root_h;
         if (!(ad->orient == _ELM_TICKERNOTI_ORIENT_BOTTOM))
           *x = root_w - ad->noti_height;
         break;
      case 180:
         *w = root_w;
         if (!ad->orient == _ELM_TICKERNOTI_ORIENT_BOTTOM)
           *y = root_h - ad->noti_height;
         break;
       case 0:
      default:
         *w = root_w;
         if (ad->orient == _ELM_TICKERNOTI_ORIENT_BOTTOM)
           *y = root_h - ad->noti_height;
         break;
     }
#endif
}

static void
_win_rotated(void *data)
{
   struct appdata *ad = data;
   int x = 0, y = 0, w = 0, angle = 0;

   if (!ad) return;
   angle = elm_win_rotation_get(ad->win);
   if (angle % 90) return;
   angle %= 360;
   if (angle < 0) angle += 360;
   ad->angle = angle;
   _update_geometry_on_rotation(ad, &x, &y, &w);
   evas_object_move(ad->win, x, y);
   evas_object_resize(ad->win, w, ad->noti_height);
#ifdef HAVE_ELEMENTARY_X
   _update_window_hints(ad->win);
#endif
}

static Eina_Bool
_prop_change(void *data, int type, void *event)
{
#ifdef HAVE_ELEMENTARY_X
   Ecore_X_Event_Window_Property *ev;
   struct appdata *ad = data;

   if (!ad) return ECORE_CALLBACK_PASS_ON;
   ev = event;
   if (ev->atom == ECORE_X_ATOM_E_ILLUME_ROTATE_WINDOW_ANGLE)
     {
        if (ev->win == elm_win_xwindow_get(ad->win))
          {
             _win_rotated(data);
          }
     }
   return ECORE_CALLBACK_PASS_ON;
#endif
}
static void _hide_cb(void *data, Evas_Object *obj, const char *emission, const char *source)
{
   struct appdata *ad = data;

   if (!ad) return;
   evas_object_hide(ad->win);
}

static void
_create_tickernoti(void *data)
{
   Evas_Coord w;
   char *data_win_height = NULL;

   if(!data) return ;
   struct appdata *ad = data;

   evas_object_move(ad->win, 0, 0);

   ad->layout = elm_layout_add(ad->win);
   elm_layout_file_set(ad->layout, TICKERNOTI_EDJ, ad->style);

   elm_win_resize_object_add(ad->win, ad->layout);
   evas_object_size_hint_weight_set(ad->layout, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);

   // tickernoti height setting
   if ((ad->style) && (!strcmp(ad->style, "info")))
      data_win_height = TICKERNOTI_INFO_BG_MIN_HEIGHT;
   else
      data_win_height = TICKERNOTI_DEFAULT_BG_MIN_HEIGHT;

   if (data_win_height != NULL && elm_config_scale_get() > 0.0)
     ad->noti_height = (int)(elm_config_scale_get() * atoi(data_win_height));

   ecore_x_window_size_get(ecore_x_window_root_first_get(), &w, NULL);
   evas_object_size_hint_min_set(ad->layout, w, ad->noti_height);
   evas_object_resize(ad->win, w, ad->noti_height);

   ad->rotation_event_handler = ecore_event_handler_add(
            ECORE_X_EVENT_WINDOW_PROPERTY, _prop_change, ad);

   edje_object_signal_callback_add(elm_layout_edje_get(ad->layout), "request,hide", "", _hide_cb, ad);
   evas_object_show(ad->layout);
}

static void
_show_tickernoti(void *data)
{
   if(!data) return ;
   struct appdata *ad = data;

   _update_window_hints(ad->win);
   evas_object_show(ad->win);
   edje_object_signal_emit(elm_layout_edje_get(ad->layout), "effect,show", "elm");
   edje_object_message_signal_process(elm_layout_edje_get(ad->layout));
}

static void
_hide(void *data, Evas *e, Evas_Object *obj, void *event_info)
{
   if(!data) return ;
   struct appdata *ad = data;
   evas_object_hide(ad->win);
}

EAPI Evas_Object *
_elm_tickernoti_add(void *data)
{
   Evas_Object *obj;

   if(!data) return NULL;
   struct appdata *ad = data;

   ad->win = _create_window(NULL, "noti-window");

   _create_tickernoti(ad);

   evas_object_event_callback_add(ad->layout, EVAS_CALLBACK_HIDE, _hide, ad);
   return ad->layout;
}

static void
_sizing_eval(void *data)
{
   if(!data) return ;
   struct appdata *ad = data;
   Evas_Coord minw = -1, minh = -1;

   elm_coords_finger_size_adjust(1, &minw, 1, &minh);
   edje_object_size_min_restricted_calc(elm_layout_edje_get(ad->layout), &minw, &minh, minw, minh);
}

static void
_elm_tickernoti_label_set(void *data)
{
   if(!data) return ;
   struct appdata *ad = data;
   edje_object_part_text_set(elm_layout_edje_get(ad->layout), "elm.text", ad->text);
   _sizing_eval(ad);
}

EAPI void
_elm_tickernoti_orient_set(void *data, _Elm_Tickernoti_Orient orient)
{
   Evas_Coord root_w, root_h;
   if(!data) return ;
   struct appdata *ad = data;

   if (orient >= _ELM_TICKERNOTI_ORIENT_LAST) return;

   ecore_x_window_size_get(ecore_x_window_root_first_get(), &root_w, &root_h);

   switch(orient) {
      case _ELM_TICKERNOTI_ORIENT_BOTTOM:
         evas_object_move(ad->win, 0, root_h - ad->noti_height);
         ad->orient = _ELM_TICKERNOTI_ORIENT_BOTTOM;
         break;
      case _ELM_TICKERNOTI_ORIENT_TOP:
      default:
         evas_object_move(ad->win, 0, 0);
         ad->orient = _ELM_TICKERNOTI_ORIENT_TOP;
         break;
   }
   _update_window_hints(ad->win);
}

static void load_small_popup(void *data)
{
	if(!data) return ;
	struct appdata *ad = data;
	int ret = -1;

	if (ad->win)
	{
		evas_object_del(ad->win);
		ad->win = NULL;
	}

	_elm_tickernoti_add(ad);

	ret = syspopup_create(ad->b, &handler, ad->win, ad);
	if(ret != 0) return;

	_elm_tickernoti_label_set(ad);
	_elm_tickernoti_orient_set(ad, ad->orient);
	evas_object_show(ad->layout);
	_show_tickernoti(ad);

	if (ad->timeout <= 0) return;
	ad->e_timer = ecore_timer_add(ad->timeout, timeout_cb, NULL);
}

static int __app_reset(bundle *b, void *data)
{

	if(!data) return -1;
	struct appdata *ad = data;
	char key[SYSPOPUP_PARAM_LEN];
	int ret = 0;

	ad->b = bundle_dup(b);

	/* When syspopup is already loaded, remove the popup and load new popup */
	if (syspopup_has_popup(b))
	{
		if (ad->e_timer)
		{
			ecore_timer_del(ad->e_timer);
			ad->e_timer = NULL;
		}
		if (ad->layout)
		{
			evas_object_del(ad->layout);
			ad->layout = NULL;
		}
		if (ad->win)
		{
			evas_object_del(ad->win);
			ad->win = NULL;
		}
		/* Resetting all proporties of syspopup */
		syspopup_reset(b);
	}

	/* get text */
	snprintf(key, SYSPOPUP_PARAM_LEN, "%d", SYSPOPUP_TEXT);
	ad->text = bundle_get_val(b, (const char *)key);
	if (!ad->text)
	{
		elm_exit();
	}

	/* get tickernoti style */
	snprintf(key, SYSPOPUP_PARAM_LEN, "%d", SYSPOPUP_STYLE);
	ad->style = bundle_get_val(b, (const char *)key);
	if (!ad->style) ad->style = "default";

	/* get orient */
	snprintf(key, SYSPOPUP_PARAM_LEN, "%d", SYSPOPUP_ORIENT);
	const char *orient = bundle_get_val(b, (const char *)key);
	if (!orient)
	{
		/* default orient */
		ad->orient = _ELM_TICKERNOTI_ORIENT_TOP;
	}
	else
	{
		ad->orient = atoi(orient);
	}

	/* get timeout */
	snprintf(key, SYSPOPUP_PARAM_LEN, "%d", SYSPOPUP_TIMEOUT);
	const char *timeout = bundle_get_val(b, (const char *)key);
	if (!timeout)
	{
		/* default timeout */
		ad->timeout = -1;
	}
	else
	{
		ad->timeout = atoi(timeout);
	}

   load_small_popup(ad);

	return 0;
}

int main(int argc, char *argv[])
{

	struct appdata ad;
	struct appcore_ops ops = {
		.create = __app_create,
		.terminate = __app_terminate,
		.pause = __app_pause,
		.resume = __app_resume,
		.reset = __app_reset,
	};

	memset(&ad, 0x0, sizeof(struct appdata));
	ops.data = &ad;

	return appcore_efl_main(PACKAGE, &argc, &argv, &ops);
}

